/*
 * audioreceiver.c
 */

#include "audioreceiver.h"
#include "postproc.h"
#include "audiorecorder.h"

#include <vdr/epg.h>
#include <vdr/plugin.h>

#include <unistd.h>
#include <stdio.h>
#include <time.h>


/* --- cAudioReceiver ------------------------------------------------------- */

cAudioReceiver::cAudioReceiver(const cChannel *chan) 
:cReceiver(0, -2, chan->Apid(0)), cThread()
{
        channel = chan;
        
        active = false;
        pes_sync = false;
	device_number = -1;
	recstat = recWait;
        
        track = NULL;
        postdata = NULL;
        rds = NULL;
        buffer = NULL;
        
        dsyslog("[audiorecorder]: receiver for channel <%s> created on pid %d "
                "(%s, %s())", channel->Name(), channel->Apid(0), __FILE__,
                __func__);
}


cAudioReceiver::~cAudioReceiver()
{
        DELETE(track);
        DELETE(postdata);
        DELETE(rds);
        DELETE(buffer);
}


void cAudioReceiver::Receive(uchar *data, int length)
{
        int ts_header_len, header_len, data_len, rb_len;
        
        if (! active || data[0] != 0x47)
                return;
        
        /* check adaptation field */ 
        switch (data[3] & 0x30) {
        case 0x10:
                /* no adaptation field, payload only */
                ts_header_len = 4;
                break;
        case 0x30:
                /* adaptation field followed by payload */
                ts_header_len = data[4] + 5;
                break;
        default:
                return;
        }
        
        header_len = ts_header_len;
        
        /* check payload unit start indicator */
        if ((data[1] & 0x40) == 0x40) {
                /* ts-header is followed by a pes-header */
                header_len  += data[ts_header_len + 8] + 9;
                pes_sync = true;
        }
        
        if (! pes_sync)
                return;
        
        /* put mp2-data into the ringbuffer */
        data_len = length - header_len;
        rb_len = buffer->Put(data + header_len, data_len);
        
        if (data_len != rb_len) {
                buffer->ReportOverflow(data_len - rb_len);
                dsyslog("[audiorecorder]: buffer overflow (%s, %s())", __FILE__,
                        __func__);
                buffer->Clear();
        }
}


void cAudioReceiver::Activate(bool on)
{
        if (on) {
                if (! active)
                        Start();
        }
        else if (active) {
                active = false;
                Cancel(3);
        }
}


void cAudioReceiver::Action(void)
{
        abuffer input_buf;
        
        dsyslog("[audiorecorder]: receiving from channel <%s> (%s, %s())",
                channel->Name(), __FILE__, __func__);
        active = true;
        
        if (buffer)
                buffer->Clear();
        else
                buffer = new cRingBufferLinear(KB(100), KB(5), true, NULL);
        
        track = new cTrack();
        postdata = new cPostData();
        rds = new cRds(channel->Name(), postdata);
        
        while (active) {
                input_buf.data = buffer->Get(input_buf.length);
                input_buf.offset = 0;
                
                if (! input_buf.data) {
                        usleep(10000);
                        continue;
                }
                
                get_mpa_frame(&input_buf, &mpa_frame, channel->Name());
                
                while (mpa_frame.data) {
                        set_recstat_rds();
                        
                        track->write_data(mpa_frame.data, mpa_frame.length);
                        
                        input_buf.offset += mpa_frame.length;
                        get_mpa_frame(&input_buf, &mpa_frame, channel->Name());
                }
                
                buffer->Del(input_buf.offset);
        }
       
	pes_sync = false;
        device_number = -1;
	
	recstat = recWait;
        
        DELETE(track);
        DELETE(postdata);
        DELETE(rds);
        
        dsyslog("[audiorecorder]: stopped receiving from channel <%s> "
                "(%s, %s())", channel->Name(), __FILE__, __func__);
}


bool cAudioReceiver::is_attached(int attached_device_number)
{
        if (device_number < 0)
                return false;
        
        if (attached_device_number != device_number &&
	    attached_device_number != -1)
		return false;
        
        return true;
}


bool cAudioReceiver::is_recording()
{
        if (recstat != recNew && recstat != recRun)
                return false;
        
        return true;
}


void cAudioReceiver::set_recstat_rds()
{
        rds->put_data(mpa_frame.data, mpa_frame.length);
        
        while(rds->set_next_frame()) {
                recstat = rds->decode_frame();
                
                control_track();
        }
}


void cAudioReceiver::control_track()
{
        switch (recstat) {
        case recNew:
                char *path, *comment;
                
                path = get_path();
                comment = get_comment();
                
                track->new_track(path);
                postdata->new_track(path, comment);
                
                FREE(path);
                FREE(comment);
                
                break;
        case recStop:
                track->stop_track();
                
                if (! postdata->get_artist() || ! postdata->get_title())
                        track->delete_track();
                else {
                        postdata->stop_track(mpa_frame.bit_rate,
                                mpa_frame.sample_rate, mpa_frame.channels,
                                mpa_frame.length);
                        
                        Postproc.put_postdata(postdata);
                }
                
                break;
        default:
                break;
        }
}


char *cAudioReceiver::get_path(void)
{
        char *path;
        time_t now;
        struct tm *tm_now, tm_store;
        char time_str[20];
        
        time(&now);
        tm_now = localtime_r(&now, &tm_store);
        
        strftime(time_str, 20, "%Y-%m-%d.%H.%M.%S", tm_now);
        
	asprintf(&path, "%s/%s-%s.tmp.mp2", cPluginAudiorecorder::get_recdir(),
                *(channel->GetChannelID()).ToString(), time_str);
        
        return path;
}


char *cAudioReceiver::get_comment(void)
{
        char *comment;
        
        const char *ev_title = NULL;
	cSchedulesLock sched_lock;
        
	const cSchedules *schedules = cSchedules::Schedules(sched_lock);
	if (! schedules)
                return NULL;
        
    	const cSchedule *sched = schedules->GetSchedule(channel);
	if (! sched)
                return NULL;
        
      	const cEvent *event = sched->GetPresentEvent();
        if (! event)
                return NULL;
        
	ev_title = event->Title();
        
        cPlugin *plugin = cPluginManager::GetPlugin("audiorecorder");
        
	asprintf(&comment, "recorded on \"%s%s%s\" (vdr-audiorecorder %s)",
                 ev_title ? ev_title : "", ev_title ? "@" : "", channel->Name(),
		 plugin->Version());
        
        return comment;
}
