/*
 * audiorecorder.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */


#include "audiorecorder.h"
#include "setup.h"
#include "postproc.h"
#include "service.h"
#include "a-tools.h"

#include <vdr/plugin.h>

#include <getopt.h>
#include <sys/stat.h>

extern "C" {
#include <ffmpeg/avcodec.h>
}


const char *cPluginAudiorecorder::DESCRIPTION = "floods your disc with music";
const char *cPluginAudiorecorder::VERSION = "0.1.0-pre4";

const char *cPluginAudiorecorder::recdir = NULL;
int cPluginAudiorecorder::debug = 0;


cPluginAudiorecorder::cPluginAudiorecorder(void)
{
	dispatcher = NULL;
        mm_entry = NULL;
}


cPluginAudiorecorder::~cPluginAudiorecorder()
{
	DELETE(dispatcher);
        FREE(mm_entry);
}


const char *cPluginAudiorecorder::CommandLineHelp(void)
{
        return "  -r dir,   --recdir=dir    set recording-directory to 'dir'.\n"
               "  -d level, --debug=level   set debugging level (default: 0),\n"
               "                            all stuff is written to stdout.\n"
               "                            0 = off\n"
               "                            1 = only errors\n"
               "                            2 = errors and further infos\n";
}


bool cPluginAudiorecorder::ProcessArgs(int argc, char *argv[])
{
        int c, option_index = 0;
        struct stat dir;
        
        static struct option long_options[] = {
                { "recdir", required_argument, NULL, 'r' },
                { "debug", required_argument, NULL, 'd' },
                { NULL }
        };
        
        c = getopt_long(argc, argv, "r:v:d:", long_options, &option_index);
        while (c != -1) {
                switch (c) {
                case 'r':
                        recdir = optarg;
                        break;
                case 'd':
                        if (atoi(optarg) > 0)
                                debug = atoi(optarg);
                        break;
                default:
                        return false;
                }
                
                c = getopt_long(argc, argv, "r:v:d:", long_options,
                        &option_index);
        }
        
        if (! recdir) {
                fprintf(stderr, "\n[audiorecorder]: parameter --recdir|-r not "
                        "given\n");
                dsyslog("[audiorecorder]: parameter --recdir|-r not given");                
                return false;
        }
        
        stat(recdir, &dir);
        
        if (! (dir.st_mode & S_IFDIR)) {
                fprintf(stderr, "\n[audiorecorder]: %s (given with parameter "
                        "--recdir|-r) is no a directory\n", recdir);
                dsyslog("[audiorecorder]: %s (given with parameter --recdir|-r)"
                        " is no a directory", recdir);
                return false;
        }
        
        if (access(recdir, R_OK | W_OK) != 0) {
                fprintf(stderr, "\n[audiorecorder]: can't access %s (given with"
                        " parameter --recdir|-r)\n", recdir);
                dsyslog("[audiorecorder]: can't access %s (given with parameter"
                        " --recdir|-r)", recdir);
                return false;
        }
        
        return true;
}


bool cPluginAudiorecorder::Initialize(void)
{
        /* Initialize any background activities the plugin shall perform. */
        return true;
}


bool cPluginAudiorecorder::Start(void)
{
        /* initialize libavcodec */
        avcodec_init();
        avcodec_register_all();
        
        probe_audio_codecs();
        
	dispatcher = new cDispatcher();
        
        return true;
}


void cPluginAudiorecorder::Stop(void)
{
        /* Stop any background activities the plugin shall perform. */
}


void cPluginAudiorecorder::Housekeeping(void)
{
        /* Perform any cleanup or other regular tasks. */
}


const char *cPluginAudiorecorder::MainMenuEntry(void)
{
        FREE(mm_entry);
        asprintf(&mm_entry, "Audiorecorder (%s): %d/%d/%d, %d",
		dispatcher->is_active() ? "on" : "off",
                dispatcher->get_recording_receivers(),
                dispatcher->get_attached_receivers(-1),
                SetupValues.max_receivers,
                Postproc.get_num_queued());
        
        if (! mm_entry)
                return NULL;
        
        return mm_entry;
}


cOsdObject *cPluginAudiorecorder::MainMenuAction(void)
{
	dispatcher->toggle();
        return NULL;
}


cMenuSetupPage *cPluginAudiorecorder::SetupMenu(void)
{
        return new cAudiorecorderSetup();
}


bool cPluginAudiorecorder::SetupParse(const char *name, const char *value)
{
        if (strcmp(name, "max_receivers") == 0)
                SetupValues.max_receivers = atoi(value);
        else if (strcmp(name, "fade_in") == 0)
                SetupValues.fade_in = atoi(value);
        else if (strcmp(name, "fade_in_mode") == 0)
                SetupValues.fade_in_mode = atoi(value);
        else if (strcmp(name, "fade_out") == 0)
                SetupValues.fade_out = atoi(value);
        else if (strcmp(name, "fade_out_mode") == 0)
                SetupValues.fade_out_mode = atoi(value);
        else if (strcmp(name, "audio_codec") == 0) {
                int c;
                
                for (c = 0; c < SetupValues.num_audio_codecs; c++) {
                        if (strcmp(value, audio_codecs[c]) == 0) {
                                SetupValues.audio_codec = c;
                                break;
                        }
                }
        }
        else if (strcmp(name, "bit_rate") == 0)
                SetupValues.bit_rate = atoi(value);
        else
                return false;
        
        
        return true;
}


bool cPluginAudiorecorder::Service(const char *id, void *data)
{
        if (! id)
                return false;
        
        if (strcmp(id, "Audiorecorder-StatusRtpChannel-v1.0") == 0) {
                if (data) {
                        struct Audiorecorder_StatusRtpChannel_v1_0 *d;
                        d = (struct Audiorecorder_StatusRtpChannel_v1_0 *)data;
                        d->status =
                                dispatcher->get_recording_status(d->channel);
                }
                return true;
        }
        
        dsyslog("[audiorecorder]: unknown service-id %s (%s, %s())", id,
                __FILE__, __func__);
        
        return false;
}

const char **cPluginAudiorecorder::SVDRPHelpPages(void)
{
        /* Return help text for SVDRP commands this plugin implements */
        return NULL;
}


cString cPluginAudiorecorder::SVDRPCommand(const char *command,
        const char *option, int &reply_code)
{
        /* Process SVDRP commands this plugin implements */
        return NULL;
}


void cPluginAudiorecorder::probe_audio_codecs() {
        int c;
        char *tmp;
        AVCodec *codec = NULL;
        
        for (c = 1; c < SetupValues.num_audio_codecs; c++) {
                codec = avcodec_find_encoder_by_name(audio_codecs[c]);
                if (codec)
                        continue;
                
                dsyslog("[audiorecorder]: your version of libavcodec (ffmpeg)"
                        "(ffmpeg) is not compiled with %s support (%s, %s())", audio_codecs[c], __FILE__, __func__);
                
                tmp = audio_codecs[c];
                audio_codecs[c] =
                        audio_codecs[SetupValues.num_audio_codecs - 1];
                audio_codecs[SetupValues.num_audio_codecs - 1] = tmp;
                SetupValues.num_audio_codecs--;
        }
}


VDRPLUGINCREATOR(cPluginAudiorecorder); /* Don't touch this! */
