/*
 * dispatcher.c
 */

#include "dispatcher.h"
#include "setup.h"
#include "audiorecorder.h"
#include "a-tools.h"

#include <vdr/device.h>
#include <vdr/channels.h>

#include <unistd.h>
#include <sys/statfs.h>


#define MIN_FREE_DISC_SPACE MB(50)

static const char *rds_channels[] = {
        "S19.2E-1-1093-28406", /* DAS MODUL */
        "S19.2E-1-1093-28475", /* Eins Live */
        "S19.2E-1-1093-28476", /* WDR 2 */
        "S19.2E-1-1093-28423", /* YOU FM */
        "S19.2E-1-1093-28468", /* SWR 3 */
        "S19.2E-1-1093-28450", /* Bremen Vier */
        "S19.2E-1-1093-28402", /* Bayern 3 */
        "S19.2E-1-1093-28437", /* NDR 2 */
        "S19.2E-1-1093-28440", /* N-JOY */
        "S19.2E-1-1093-28421", /* hr3 */
        "S19.2E-1-1093-28420", /* hr2 */        
        "S19.2E-1-1093-28419", /* hr1 */
};

static const int num_rds_channels = 12;

/* --- cDispatcher----------------------------------------------------------- */

cDispatcher::cDispatcher(void)
:cThread(), cStatus()
{
        int c;
        tChannelID channel_id;
        cChannel *channel;
        
        active = false;
        audioreceivers = NULL;
        
        /* initialize one receiver for each channel */
        audioreceivers = new (cAudioReceiver *)[num_rds_channels];
        for (c = 0; c < num_rds_channels; c++) {
                channel_id = tChannelID::FromString(rds_channels[c]);
                
                if (! channel_id.Valid()) {
                        dsyslog("[audiorecorder]: channel %s is not valid (%s, "
                                "%s())", rds_channels[c], __FILE__, __func__);
                        continue;
                }
                
                channel = Channels.GetByChannelID(channel_id);
                if (channel)
                        audioreceivers[c] = new cAudioReceiver(channel);
                else
                        audioreceivers[c] = NULL;
        }
        
        Activate(true);
}


cDispatcher::~cDispatcher()
{
        int c;
        
        if (audioreceivers) {
                for (c = 0; c < num_rds_channels; c++)
                        DELETE(audioreceivers[c]);
        }
        
        DELETEA(audioreceivers);
}


void cDispatcher::Activate(bool on)
{
        if (on) {
                if (! active)
                        Start();
        }
        else if (active) {
                active = false;
                Cancel(3);
        }
}


void cDispatcher::Action(void)
{
        int receivers;
        active = true;
        
        dsyslog("[audiorecorder]: dispatcher thread started (%s, %s())",
                __FILE__, __func__);
        
        while (active) {
                sleep(2);
                
                if (! check_free_disc_space()) {
                        dsyslog("[audiorecorder]: not enough free disc space on"
                                " %s, recording will be stopped (%s, %s())",
                                cPluginAudiorecorder::get_recdir(), __FILE__,
                                __func__);
                        
                        active = false;
                        continue;
                }
                
                receivers = get_attached_receivers(-1);
                if (SetupValues.max_receivers > receivers)
                        attach_receivers();
                else if (SetupValues.max_receivers < receivers)
                        detach_receivers(-1, SetupValues.max_receivers);
        }
        
        detach_receivers(-1, 0);
        
        dsyslog("[audiorecorder]: dispatcher thread stopped (%s, %s())",
                __FILE__, __func__);
}


void cDispatcher::ChannelSwitch(const cDevice *device, int channel_number)
{
	/*
	 * workaround to detach active audioreceivers if the attached device
	 * is switched to another transponder. this should be done inside of
	 * vdr, but there are situations which lead to emergency exits of vdr
	 * without this peace of code (for example if a recording starts).
         *
	 */
         
        int device_number;
        
        
        if (channel_number == 0)
                return;
        
        device_number = device->DeviceNumber();
        
        if (get_attached_receivers(device_number) == 0)
                return;
        
	if (Channels.GetByNumber(channel_number)->Transponder() ==
	    get_transponder_of_first_receiver(device_number))
		return;
        
        detach_receivers(device_number, 0);
}


void cDispatcher::toggle(void)
{
        if (active)
                Activate(false);
        else
                Activate(true);
}


int cDispatcher::get_attached_receivers(int device_number)
{
        int c, count = 0;
        
        if (! audioreceivers)
                return 0;
        
        for (c = 0; c < num_rds_channels; c++) {
                if (! audioreceivers[c])
                        continue;
                
                if (audioreceivers[c]->is_attached(device_number))
			count++;
        }
       	
        return count;
}


int cDispatcher::get_recording_receivers(void)
{
        int c, count = 0;
        
        if (! audioreceivers)
                return 0;
        
        for (c = 0; c < num_rds_channels; c++) {
                if (audioreceivers[c] && audioreceivers[c]->is_recording())
                        count++;
        }
        
        return count;
}


int cDispatcher::get_recording_status(const cChannel *channel)
{
        int c;
        
        if (! audioreceivers)
                return 0;
        
        for (c = 0; c < num_rds_channels; c++) {
                if (audioreceivers[c] &&
                    audioreceivers[c]->get_channel() == channel) {
                        if (audioreceivers[c]->is_recording())
                                return 3;
                        else if (audioreceivers[c]->is_attached(-1))
                                return 2;
                        else
                                return 1;
                    }
        }
        
        return 0;
}


void cDispatcher::detach_receivers(int detach_device_number,
        int devices_remaining)
{
        int c;
        int device_number;
        cDevice *device = NULL;
        
        if (! audioreceivers)
                return;
        
        Lock();
        
        for (c = num_rds_channels - 1; c > -1; c--) {
                if (get_attached_receivers(-1) <= devices_remaining)
                        break;
                
                if (! audioreceivers[c])
                        continue;
                
                device_number = audioreceivers[c]->get_device_number();
                
                if (device_number < 0)
                        continue;
                
                if (detach_device_number != device_number &&
                    detach_device_number > 0)
                        continue;
                
                device = cDevice::GetDevice(device_number);
                
                if (! device)
                        continue;
                
                device->Detach(audioreceivers[c]);
        }
        
        Unlock();
}


void cDispatcher::attach_receivers(void)
{
        int c;
        cAudioReceiver *receiver;
        cDevice *device;
        bool needs_detach;
        
        Lock();
        
        for (c = 0; c < num_rds_channels; c++) {
                if (get_attached_receivers(-1) >= SetupValues.max_receivers)
                        break;
                
                receiver = audioreceivers[c];
                
                if (! receiver || receiver->is_attached(-1))
                        continue;
                
                device = cDevice::GetDevice(receiver->get_channel(), 0,
                        &needs_detach);
                
                if (! device || (device && needs_detach))
                        continue;
                
#ifndef NO_ACTUAL_DEVICE
		if (device == cDevice::ActualDevice() &&
                    ! device->IsTunedToTransponder(receiver->get_channel()))
                        continue;
#endif /* NO_ACTUAL_DEVICE */
                
                if (! device->IsTunedToTransponder(receiver->get_channel()) &&
                    ! device->SwitchChannel(receiver->get_channel(), false))
                        continue;
                
                receiver->set_device_number(device->DeviceNumber());
                device->AttachReceiver(receiver);
        }
        
        Unlock();
}


int cDispatcher::get_transponder_of_first_receiver(int device_number)
{
        int c, transponder = 0;
        
        if (! audioreceivers)
                return 0;
        
        for (c = 0; c < num_rds_channels; c++) {
                if (! audioreceivers[c])
                        continue;
                
                if (audioreceivers[c]->is_attached(device_number)) {
                        transponder =
                                audioreceivers[c]->get_channel()->Transponder();
                        break;
                }
        }
        
        return transponder;
}


bool cDispatcher::check_free_disc_space(void)
{
        struct statfs stat_fs;
        
        if (statfs(cPluginAudiorecorder::get_recdir(), &stat_fs) != 0)
                return false;
        
        if (stat_fs.f_bavail * stat_fs.f_bsize < MIN_FREE_DISC_SPACE)
                return false;
        
        return true;
}
